@@ -4,6 +4,7 @@ import os
 import signal
 import sys
 from smot.util import die
+import smot.format as sf
 
 INT_SENTINEL = 9999
 
@@ -75,7 +76,7 @@ ListOfStrings = ListOfStringsType()
 
 
 def factorTree(
-    tree,
+    node,
     factor_by_capture=None,
     factor_by_field=None,
     factor_by_table=None,
@@ -90,20 +91,20 @@ def factorTree(
             die(
                 f"""Expected a positive integer for field --factor-by-field, got '{factor_by_field}'"""
             )
-        tree = alg.factorByField(tree, field, default=default)
+        nodes = alg.factorByField(node, field, default=default)
     elif factor_by_capture is not None:
-        tree = alg.factorByCapture(tree, pat=factor_by_capture, default=default)
+        node = alg.factorByCapture(node, pat=factor_by_capture, default=default)
     elif factor_by_table is not None:
-        tree = alg.factorByTable(tree, filename=factor_by_table, default=default)
-    tree = alg.setFactorCounts(tree)
-    return tree
+        node = alg.factorByTable(node, filename=factor_by_table, default=default)
+    node = alg.setFactorCounts(node)
+    return node
 
 
 def read_tree(treefile):
     from smot.parser import p_tree
 
     rawtree = treefile.readlines()
-    rawtree = "\n".join(rawtree).strip()
+    rawtree = "".join(rawtree)
     tree = p_tree.parse(rawtree)
     return tree
 
@@ -111,17 +112,6 @@ def read_tree(treefile):
 dec_tree = click.argument("TREE", default=sys.stdin, type=click.File())
 
 
-#      smot convert --from=<from> --to=<to> [<filename>]
-@click.command(help="Convert between tree formats.")
-@click.option("--from", "opt_from", type=str, help="input tree format")
-@click.option("--to", "opt_to", type=str, help="output tree format")
-@dec_tree
-def convert(opt_from, opt_to, tree):
-    from Bio import Phylo
-
-    Phylo.convert(tree, opt_from, sys.stdout, opt_to)
-
-
 #      smot tips [<filename>]
 @click.command(help="Print the tree tip labels")
 @dec_tree
@@ -129,29 +119,17 @@ def tips(tree):
     import smot.algorithm as alg
 
     tree = read_tree(tree)
-    tree = alg.setNLeafs(tree)
+    tree.tree = alg.setNLeafs(tree.tree)
 
     def _fun(b, x):
         if x.isLeaf:
             b.append(x.label)
         return b
 
-    for tip in alg.treefold(tree, _fun, []):
+    for tip in alg.treefold(tree.tree, _fun, []):
         print(tip)
 
 
-#      smot plot [<filename>]
-@click.command(help="Build a simple tree plot")
-@dec_tree
-def plot(tree):
-    from Bio import Phylo
-
-    tree = read_tree(tree)
-    btree = Phylo.BaseTree.Tree.from_clade(tree.asBiopythonTree())
-    btree.ladderize(reverse=True)
-    Phylo.draw(btree)
-
-
 def factoring(function):
     function = click.option(
         "--factor-by-capture",
@@ -180,6 +158,12 @@ dec_proportion = click.option(
     help="The proportion of tips in a clade to keep",
 )
 
+dec_newick = click.option(
+    "--newick",
+    is_flag=True,
+    help="Write output in simple newick format (tip colors and metadata will be lost)",
+)
+
 dec_scale = click.option(
     "-s",
     "--scale",
@@ -208,7 +192,11 @@ dec_keep = click.option(
 )
 
 dec_keep_regex = click.option(
-    "-r", "--keep-regex", default="", type=str, help="Keep all tips that match this pattern, these tips do not count towards downsampling quotas."
+    "-r",
+    "--keep-regex",
+    default="",
+    type=str,
+    help="Keep all tips that match this pattern, these tips do not count towards downsampling quotas.",
 )
 
 
@@ -222,6 +210,7 @@ dec_keep_regex = click.option(
 )
 @dec_max_tips
 @click.option("--zero", is_flag=True, help="Set branches without lengths to 0")
+@dec_newick
 @dec_tree
 def equal(
     factor_by_capture,
@@ -231,20 +220,25 @@ def equal(
     default,
     max_tips,
     zero,
+    newick,
     tree,
 ):
     import smot.algorithm as alg
 
     tree = read_tree(tree)
-    tree = factorTree(
-        tree=tree,
+    tree.tree = factorTree(
+        node=tree.tree,
         factor_by_capture=factor_by_capture,
         factor_by_field=factor_by_field,
         factor_by_table=factor_by_table,
         default=default,
     )
-    tree = alg.sampleContext(tree, keep=keep, maxTips=max_tips)
-    print(tree.newick())
+    tree.tree = alg.sampleContext(tree.tree, keep=keep, maxTips=max_tips)
+
+    if newick:
+        print(sf.newick(tree))
+    else:
+        print(sf.nexus(tree))
 
 
 @click.command(
@@ -260,6 +254,7 @@ def equal(
 @dec_proportion
 @dec_scale
 @dec_seed
+@dec_newick
 @click.option("--zero", is_flag=True, help="Set branches without lengths to 0")
 @dec_tree
 def prop(
@@ -273,6 +268,7 @@ def prop(
     proportion,
     scale,
     seed,
+    newick,
     zero,
     tree,
 ):
@@ -282,17 +278,27 @@ def prop(
         die("Please add either a --proportion or --scale option")
 
     tree = read_tree(tree)
-    tree = factorTree(
-        tree=tree,
+    tree.tree = factorTree(
+        node=tree.tree,
         factor_by_capture=factor_by_capture,
         factor_by_field=factor_by_field,
         factor_by_table=factor_by_table,
         default=default,
     )
-    tree = alg.sampleProportional(
-        tree, keep=keep, keep_regex=keep_regex, proportion=proportion, scale=scale, minTips=min_tips, seed=seed
+    tree.tree = alg.sampleProportional(
+        tree.tree,
+        keep=keep,
+        keep_regex=keep_regex,
+        proportion=proportion,
+        scale=scale,
+        minTips=min_tips,
+        seed=seed,
     )
-    print(tree.newick())
+
+    if newick:
+        print(sf.newick(tree))
+    else:
+        print(sf.nexus(tree))
 
 
 @click.command(
@@ -308,6 +314,7 @@ def prop(
 @dec_proportion
 @dec_scale
 @dec_seed
+@dec_newick
 @click.option("--zero", is_flag=True, help="Set branches without lengths to 0")
 @dec_tree
 def para(
@@ -321,6 +328,7 @@ def para(
     proportion,
     scale,
     seed,
+    newick,
     zero,
     tree,
 ):
@@ -330,17 +338,27 @@ def para(
         die("Please add either a --proportion or --scale option")
 
     tree = read_tree(tree)
-    tree = factorTree(
-        tree=tree,
+    tree.tree = factorTree(
+        node=tree.tree,
         factor_by_capture=factor_by_capture,
         factor_by_field=factor_by_field,
         factor_by_table=factor_by_table,
         default=default,
     )
-    tree = alg.sampleParaphyletic(
-        tree, keep=keep, keep_regex=keep_regex, proportion=proportion, scale=scale, minTips=min_tips, seed=seed
+    tree.tree = alg.sampleParaphyletic(
+        tree.tree,
+        keep=keep,
+        keep_regex=keep_regex,
+        proportion=proportion,
+        scale=scale,
+        minTips=min_tips,
+        seed=seed,
     )
-    print(tree.newick())
+
+    if newick:
+        print(sf.newick(tree))
+    else:
+        print(sf.nexus(tree))
 
 
 @click.command(
@@ -368,6 +386,7 @@ def para(
     default=False,
     help="Infer factors by distance on the tree to the nearest label",
 )
+@dec_newick
 @dec_tree
 def factor(
     method,
@@ -377,13 +396,14 @@ def factor(
     default,
     impute,
     patristic,
+    newick,
     tree,
 ):
     import smot.algorithm as alg
 
     tree = read_tree(tree)
-    tree = factorTree(
-        tree=tree,
+    tree.tree = factorTree(
+        node=tree.tree,
         factor_by_capture=factor_by_capture,
         factor_by_field=factor_by_field,
         factor_by_table=factor_by_table,
@@ -391,9 +411,9 @@ def factor(
     )
 
     if patristic:
-        tree = alg.imputePatristicFactors(tree)
+        tree.tree = alg.imputePatristicFactors(tree.tree)
     elif impute:
-        tree = alg.imputeFactors(tree)
+        tree.tree = alg.imputeFactors(tree.tree)
 
     # create TAB-delimited, table with columns for the tip labels and the
     # (possibly imputed) factor
@@ -408,7 +428,7 @@ def factor(
                 b.append(f"{x.label}\t{factor}")
             return b
 
-        for row in alg.treefold(tree, _fun, []):
+        for row in alg.treefold(tree.tree, _fun, []):
             print(row)
 
     # prepend or append the factor to the tip labels and print the resulting tree
@@ -424,16 +444,21 @@ def factor(
                     x.label = f"{x.label}|{x.factor}"
             return x
 
-        tree = alg.treemap(tree, _fun)
-        print(tree.newick())
+        tree.tree = alg.treemap(tree.tree, _fun)
+
+        if newick:
+            print(sf.newick(tree))
+        else:
+            print(sf.nexus(tree))
 
 
 #      smot tipsed <pattern> <replacement> [<filename>]
 @click.command(help="Search and replace patterns in tip labels")
 @click.argument("PATTERN", type=str)
 @click.argument("REPLACEMENT", type=str)
+@dec_newick
 @dec_tree
-def tipsed(pattern, replacement, tree):
+def tipsed(pattern, replacement, newick, tree):
     import smot.algorithm as alg
     import re
 
@@ -445,8 +470,12 @@ def tipsed(pattern, replacement, tree):
         return nodeData
 
     tree = read_tree(tree)
-    tree = alg.treemap(tree, fun_)
-    print(tree.newick())
+    tree.tree = alg.treemap(tree.tree, fun_)
+
+    if newick:
+        print(sf.newick(tree))
+    else:
+        print(sf.nexus(tree))
 
 
 @click.command(help="Prune a tree to preserve only the tips with that match a pattern")
@@ -463,8 +492,9 @@ def tipsed(pattern, replacement, tree):
     is_flag=True,
     help="Read patterns from a file instead of a set string",
 )
+@dec_newick
 @dec_tree
-def grep(pattern, tree, invert_match, perl, file):
+def grep(pattern, tree, invert_match, perl, newick, file):
     import smot.algorithm as alg
     import re
 
@@ -490,38 +520,151 @@ def grep(pattern, tree, invert_match, perl, file):
         ]
 
     tree = read_tree(tree)
-    tree = alg.treecut(tree, fun_)
-    tree = alg.clean(tree)
-    print(tree.newick())
+    tree.tree = alg.clean(alg.treecut(tree.tree, fun_))
+
+    if newick:
+        print(sf.newick(tree))
+    else:
+        print(sf.nexus(tree))
 
 
-@click.command(help="Generate a random tree from a file of labels")
-@dec_seed
-@click.argument("TIPNAMES", default=sys.stdin, type=click.File())
-def random(seed, tipnames):
-    from Bio import Phylo
-    import random
-
-    names = [name.strip() for name in tipnames.readlines()]
-    random.seed(seed)
-    btree = Phylo.BaseTree.Tree.randomized(names)
-    Phylo.write(btree, file=sys.stdout, format="newick")
-
-
-#      smot clean [<filename>]
-@click.command(
-    help="Clean and sort the tree. Nodes with single children are removed. Branch lengths are added (defaulting to 0). The tree is sorted (the topology is NOT changed and no root is added)."
+@click.command()
+@click.option("-p", "--pattern", nargs=2, multiple=True)
+@click.option(
+    "-P", "--perl", is_flag=True, help="Interpret the pattern as a regular expression"
 )
 @dec_tree
-def clean(tree):
-    from Bio import Phylo
+def leaf(pattern, perl, tree):
+    """
+    Color the tips on a tree.
+
+    smot color -p "swine" "#FFA500" -p "2020-" "#00FF00" my.tre > color.tre
+    """
+    import smot.algorithm as alg
+    import re
+
+    tree = read_tree(tree)
+
+    def _fun(b, x):
+        if x.isLeaf:
+            b.append(x.label)
+        return b
+
+    tips = alg.treefold(tree.tree, _fun, [])
+
+    for (pat_str, col) in pattern:
+        if perl:
+            pat = re.compile(pat_str)
+            matcher = lambda x: re.search(pat, x)
+        else:
+            matcher = lambda x: pat_str in x
+        for tip in tips:
+            if matcher(tip):
+                tree.colmap[tip] = col
+
+    print(sf.nexus(tree))
+
+
+colormap_arg = click.option(
+    "-c",
+    "--colormap",
+    type=click.Path(exists=True),
+    help="A TAB-delimited, headless table with columns for factor names and hexadecimal colors",
+)
+
+
+def chooseColorScheme(factors):
+    # these colors are adapted from Paul Tol's notes here: https://personal.sron.nl/~pault/#sec:qualitative
+    if len(factors) == 2:
+        # orange and blue
+        colors = ["#FFA000", "#0000FF"]
+    elif len(factors) == 3:
+        # Paul's three
+        colors = ["#004488", "#DDAA33", "#BB5566"]
+    elif len(factors) <= 6:
+        # Paul's medium-contrast 6
+        colors = ["#5486C0", "#053275", "#E9C353", "#866503", "#E7849A", "#863144"]
+    elif len(factors) <= 11:
+        # Paul's sunset
+        colors = [
+            "#2A3789",
+            "#3A65A8",
+            "#5C95C2",
+            "#87BEDA",
+            "#E5E8C0",
+            "#FCD17A",
+            "#FAA353",
+            "#F1693C",
+            "#D32623",
+            "#91001C",
+        ]
+    else:
+        die("I can't handle more than 11 colors yet")
+
+    colormap = {f : c for (f,c) in  zip(factors, colors)}
+
+    return colormap
+
+
+def colorBranches(
+    is_para, factor_by_capture, factor_by_field, factor_by_table, colormap, tree
+):
     import smot.algorithm as alg
 
     tree = read_tree(tree)
-    tree = alg.clean(tree)
-    btree = Phylo.BaseTree.Tree.from_clade(tree.asBiopythonTree())
-    btree.ladderize(reverse=True)
-    Phylo.write(btree, file=sys.stdout, format="newick")
+
+    tree.tree = factorTree(
+        node=tree.tree,
+        factor_by_capture=factor_by_capture,
+        factor_by_field=factor_by_field,
+        factor_by_table=factor_by_table,
+    )
+    tree.tree = alg.setFactorCounts(tree.tree)
+
+    factors = sorted(list(tree.tree.data.factorCount.keys()))
+
+    _colormap = dict()
+    if colormap:
+        with open(colormap, "r") as f:
+            try:
+                _colormap = {f: c for (f, c) in [p.split("\t") for p in f.readlines()]}
+            except ValueError:
+                die("Invalid color map: expected TAB-delimited, two-column file")
+    else:
+        _colormap = chooseColorScheme(factors)
+
+    if is_para:
+        tree.tree = alg.colorPara(tree.tree, colormap=_colormap)
+    else:
+        tree.tree = alg.colorMono(tree.tree, colormap=_colormap)
+
+    print(sf.nexus(tree))
+
+
+@click.command(name="mono")
+@factoring
+@colormap_arg
+@dec_tree
+def mono_color_cmd(**kwargs):
+    """
+    Color a tree by monophyletic factor.
+
+    smot color branch mono --factor-by-capture="(1B\.[^|]*)" 1B.tre
+    """
+    colorBranches(is_para=False, **kwargs)
+
+
+@click.command(name="para")
+@factoring
+@colormap_arg
+@dec_tree
+def para_color_cmd(**kwargs):
+    """
+    Color a tree by paraphyletic factor.
+
+    smot color branch para --factor-by-capture="(1B.[^|]*)" 1B.tre
+    """
+    colorBranches(is_para=True, **kwargs)
 
 
 CONTEXT_SETTINGS = dict(help_option_names=["-h", "--help"])
@@ -544,15 +687,34 @@ sample.add_command(equal)
 sample.add_command(prop)
 sample.add_command(para)
 
-cli.add_command(convert)
+
+@click.group()
+def branch():
+    "Color the branches of a tree. You may provide a color map; if you do not, smot will automatically map factors to colors from a color-blind friendly palette."
+    pass
+
+
+branch.add_command(mono_color_cmd)
+branch.add_command(para_color_cmd)
+
+
+@click.group(
+    help="Color the tips or branches. The coloring options are highly opinionated. Leaf colors are based on patterns inferred from leaf labels. They are generally independent of the leaf context within the tree. There is no direct way to color leafs by clade (and I don't think there should be). Group coloring should be done at the branch color level. Branch coloring is explicitly phylogenetic - you may color branches that monophyletic or paraphyletic for a given factor. There is no simple way to color a particular branch (and why would you want to do that anyway). So, follow my tree coloring dogma and everything will be fine.",
+    context_settings=CONTEXT_SETTINGS,
+)
+def color():
+    pass
+
+
+color.add_command(leaf)
+color.add_command(branch)
+
 cli.add_command(tips)
-cli.add_command(plot)
 cli.add_command(sample)
 cli.add_command(factor)
 cli.add_command(tipsed)
 cli.add_command(grep)
-cli.add_command(random)
-cli.add_command(clean)
+cli.add_command(color)
 
 
 def main():
